package com.gic.store.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.ClerkDTO;
import com.gic.store.dto.ClerkSearchDTO;
import com.gic.store.dto.ClerkStoreListDTO;

import java.util.List;

/**
 * @ClassName: ClerkApiService
 * @Description: 导购service 
 * @author zhiwj
 * @date 2019/7/31 11:36
 */
public interface ClerkApiService {
    
    /**
     * @Title: saveOrUpdate
     * @Description: 保存或编辑导购
     * @author zhiwj 
     * @param clerkDTO 导购
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse<Void> saveOrUpdate(ClerkDTO clerkDTO);

    /**
     * @Title: listClerk
     * @Description: 导购列表
     * @author zhiwj 
     * @param enterpriseId 企业id
     * @param clerkSearchDTO
     * @param pageNum
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.store.dto.ClerkStoreListDTO>>
     * @throws 
     */
    ServiceResponse<Page<ClerkStoreListDTO>> listClerk(Integer enterpriseId, ClerkSearchDTO clerkSearchDTO, Integer pageNum, Integer pageSize);

    /**
     * 获取clerk总数
     * @Title: getTotalClerk
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param clerkSearchDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> getTotalClerk(Integer enterpriseId, ClerkSearchDTO clerkSearchDTO);

    /**
     * 批量修改状态
     * @Title: updateClerkStatus
     * @Description: 批量修改导购状态
     * @author zhiwj 
     * @param enterpriseId 企业id
     * @param clerkIds 导购ids
     * @param status 状态
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws 
     */
    ServiceResponse<Void> updateClerkStatus(Integer enterpriseId, String clerkIds, Integer status);

    /**
     * 查询list通过id
     * @Title: listClerkByIds
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param clerkIds
     * @return java.util.List<com.gic.store.dto.ClerkDTO>
     * @throws
     */
    ServiceResponse<List<ClerkDTO>> listClerkByIds(Integer enterpriseId, String clerkIds);

    ServiceResponse<List<ClerkDTO>> listClerkByStoreId(Integer enterpriseId, Integer storeId, String search);

    ServiceResponse<ClerkDTO> getClerkById(Integer enterpriseId, Integer clerkId);

    ServiceResponse<List<ClerkDTO>> listClerk(Integer enterpriseId, String search);
}
