package com.gic.store.service;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreFieldDTO;
import com.gic.store.dto.StoreSearchDTO;

import java.util.List;

/**
 * @author zhiwj
 * @date 2019/6/24
 */
public interface StoreApiService {

    /**
     * @Title: saveOrUpdate
     * @Description: 保存门店
     * @author zhiwj
     * @param storeDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> saveOrUpdate(StoreDTO storeDTO);
    /**
     * @Title: listStore
     * @Description: 分页门店列表 查es
     * @author zhiwj 
     * @param storeSearchDTO
     * @param pageNum
     * @param pageSize
     * @param returnFileds
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws 
     */
    ServiceResponse<Page<StoreDTO>> listStore(StoreSearchDTO storeSearchDTO, Integer pageNum, Integer pageSize, String returnFileds);

    /**
     * @Title: listStore
     * @Description: 分页门店列表 查es,查询门店所有字段
     * @author zhiwj
     * @param storeSearchDTO
     * @param pageNum
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse<Page<StoreDTO>> listStore(StoreSearchDTO storeSearchDTO, Integer pageNum, Integer pageSize);
    /**
     * @Title: getStoreById
     * @Description: 查询门店详情
     * @author zhiwj 
     * @param enterpriseId
     * @param storeId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreDTO>
     * @throws 
     */
    ServiceResponse<StoreDTO> getStoreById(Integer enterpriseId, Integer storeId);

    /**
     * @Title: validateFieldExceptRepeat
     * @Description: 校验除 是否重复 外的所有条件
     * @author zhiwj 
     * @param value
     * @param storeFieldDTO
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws 
     */
    ServiceResponse<Void> validateFieldExceptRepeat(String value, StoreFieldDTO storeFieldDTO);

    /**
     * @Title: listStoreOverFlow
     * @Description: 溢出门店列表
     * @author zhiwj 
     * @param enterpriseId
     * @param page
     * @param search
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.store.dto.StoreDTO>>
     * @throws 
     */
    ServiceResponse<Page<StoreDTO>> listStoreOverFlow(Integer enterpriseId, Page<StoreDTO> page, String search);
    
    /**
     * @Title: bulkUpdateStore
     * @Description: 门店批量操作
     * @author zhiwj 
     * @param storeIds
     * @param value
     * @param enterpriseId
     * @param type
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws 
     */
    ServiceResponse bulkUpdateStore(String storeIds, String value, Integer enterpriseId, int type, Integer operatorId, String operatorName);
    
    /**
     * @Title: getByStoreNameAndStoreCode
     * @Description: 查询门店
     * @author zhiwj 
     * @param enterpriseId
     * @param storeName
     * @param storeCode
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreDTO>
     * @throws 
     */
    ServiceResponse<StoreDTO> getByStoreNameAndStoreCode(Integer enterpriseId, String storeName, String storeCode);

    /**
     * @Title: addStoreToIndex
     * @Description: 添加单个门店到es
     * @author zhiwj 
     * @param enterpriseId
     * @param storeId
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws 
     */
    ServiceResponse<Void> addStoreToIndex(Integer enterpriseId, Integer storeId);

    /**
     * @Title: bulkAddStoreToIndex
     * @Description: 批量添加门店到索引
     * @author zhiwj 
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws 
     */
    ServiceResponse<Void> bulkAddStoreToIndex(Integer enterpriseId);

    /**
     * @Title: bulkAddStoreToIndex
     * @Description: 批量添加门店到索引
     * @author zhiwj 
     * @param enterpriseId
     * @param storeIdList
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws 
     */
    ServiceResponse<Void> bulkAddStoreToIndex(Integer enterpriseId, List<Integer> storeIdList);
    
    /**
     * @Title: updateStoreToIndex
     * @Description: 批量编辑门店到索引
     * @author zhiwj 
     * @param enterpriseId
     * @param storeId
     * @param indexId
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws 
     */
    ServiceResponse<Void> updateStoreToIndex(Integer enterpriseId, Integer storeId, Long indexId);
    
    /**
     * @Title: bulkUpdateStoreToIndex
     * @Description: 批量编辑门店到索引
     * @author zhiwj 
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws 
     */
    ServiceResponse<Void> bulkUpdateStoreToIndex(Integer enterpriseId);
    
    /**
     * @Title: bulkUpdateStoreToIndex
     * @Description: 批量编辑门店到索引
     * @author zhiwj 
     * @param enterpriseId
     * @param storeIdList
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws 
     */
    ServiceResponse<Void> bulkUpdateStoreToIndex(Integer enterpriseId, List<Integer> storeIdList);
    
    /**
     * @Title: queryStoreFromES
     * @Description: 查询es
     * @author zhiwj
     * @param jsonObject
     * @param pageNum
     * @param pageSize
     * @param sortJson
     * @param returnFields
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws 
     */
    ServiceResponse<Void> queryStoreFromES(JSONObject jsonObject, Integer pageNum, Integer pageSize, JSONObject sortJson, String returnFields);

    /** @Description: 查询门店数量 from es
     * @author taogs
     * @Date 13:37 2019/9/25
     * @Param 
     * @return 
    */
    ServiceResponse<Long> queryStoreCountFromES(StoreSearchDTO storeSearchDTO);
    
    /** @Description: 删除门店 from es
     * @author taogs
     * @Date 18:44 2019/9/25
     * @Param 
     * @return 
    */
    ServiceResponse<Void> deleteStoreFromES(Integer enterpriseId, Integer storeId);
    
    /** @Description: 门店选择器参数转换
     * @author taogs
     * @Date 11:26 2019/8/23
     * @Param 
     * @return 
    */
    ServiceResponse<JSONObject> parseStoreSelectJson(String searchJson);

    ServiceResponse<List<StoreDTO>> listStoreByStoreInfoId(Integer storeInfoId);

    ServiceResponse<List<StoreDTO>> listStore(String search, Integer enterpriseId);

    
    /** @Description: 附近门店
     * @author taogs
     * @Date 19:41 2019/12/4
     * @Param 
     * @return 
    */
    ServiceResponse<Page<StoreDTO>> pageNearbyStore(Integer enterpriseId, String cityId, String longitude,
                                                    String latitude, Integer brandId, String appid, Integer pageNum,
                                                    Integer pageSize);

    ServiceResponse<Void> storeQrcodeCallBack(String param);
}
