package com.gic.store.utils.field;

import com.gic.store.constant.StoreFieldTextEnum;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zhiwj
 * @date 2019/7/5
 */
public class TextField extends FieldBase {
    private String tipMessage;
    private String textType;
    private Integer maxLength;
    private Integer minLength;

    public String getTipMessage() {
        return tipMessage;
    }

    public void setTipMessage(String tipMessage) {
        this.tipMessage = tipMessage;
    }

    public List<Integer> getTextType() {
        List<Integer> result = new ArrayList<>();
        String[] list = textType.split(",");
        for (String s : list) {
            result.add(Integer.parseInt(s));
        }
        return result;
    }

    public void setTextType(String textType) {
        this.textType = textType;
    }

    public Integer getMaxLength() {
        return maxLength;
    }

    public void setMaxLength(Integer maxLength) {
        this.maxLength = maxLength;
    }

    public Integer getMinLength() {
        return minLength;
    }

    public void setMinLength(Integer minLength) {
        this.minLength = minLength;
    }

    @Override
    public String validate(String value) {
        String msg = super.validate(value);
        if (msg != null) {
            return msg;
        }
        if (value == null || "".equals(value)) {
            return null;
        }

        // 获取没有被选择的类型，然后验证是否有这些类型， 如果出现了这些类型，就报错
        Map<Integer, StoreFieldTextEnum> textTypeList = new HashMap<>(StoreFieldTextEnum.values().length);
        for (StoreFieldTextEnum textEnum : StoreFieldTextEnum.values()) {
            textTypeList.put(textEnum.getCode(), textEnum);
        }
        String[] filedText = textType.split(",");
        for (String s : filedText) {
            textTypeList.remove(Integer.valueOf(s));
        }
        for (StoreFieldTextEnum storeFieldTextEnum : textTypeList.values()) {
            if (value.matches(storeFieldTextEnum.getPattern())) {
                return "不能出现"  + storeFieldTextEnum.getMessage() + "类型";
            }
        }
        if (value.length() < minLength) {
            return "长度过小";
        }
        if (value.length() > maxLength) {
            return "长度过大";
        }
        return null;
    }

    public static void main(String[] args) {
        TextField textField = new TextField();
        textField.setIsMust(1);
        textField.setMaxLength(30);
        textField.setMinLength(1);
        textField.setTextType("2,4");
        //中文
        //数字
        //字母
        //符号
        String 中文 = "发动机看了";
        String 数字 = "1234";
        String 字母 = "klsdf";
        String 符号 = "&^*";
        String 中文数字 = 中文 + 数字;
        String 中文字母 = 中文 + 字母;
        String 中文符号 = 中文 + 符号;
        String 数字字母 = 数字 + 字母;
        String 数字符号 = 数字 + 符号;
        String 字母符号 = 字母 + 符号;

        System.out.println("中文:" + textField.validate(中文));
        System.out.println("数字:" + textField.validate(数字));
        System.out.println("字母:" + textField.validate(字母));
        System.out.println("符号:" + textField.validate(符号));
        System.out.println("中文数字:" + textField.validate(中文数字));
        System.out.println("中文字母:" + textField.validate(中文字母));
        System.out.println("中文符号:" + textField.validate(中文符号));
        System.out.println("数字字母:" + textField.validate(数字字母));
        System.out.println("数字符号:" + textField.validate(数字符号));
        System.out.println("字母符号:" + textField.validate(字母符号));


    }

}
