package com.gic.store.service.impl;

import com.gic.commons.util.GlobalInfo;
import com.gic.store.constant.WeekdayEnum;
import com.gic.store.dao.mapper.TabStoreBusinessTimeMapper;
import com.gic.store.dto.StoreBusinessTimeDTO;
import com.gic.store.entity.TabStoreBusinessTime;
import com.gic.store.service.StoreBusinessTimeService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author zhiwj
 * @date 2019/6/28
 */
@Service
public class StoreBusinessTimeServiceImpl implements StoreBusinessTimeService {

    @Autowired
    TabStoreBusinessTimeMapper tabStoreBusinessTimeMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void editStoreBusinessTime(Integer enterpriseId, Integer storeInfoId, List<StoreBusinessTimeDTO> businessTimeList, List<Integer> delBusinessTimes) {
        if (CollectionUtils.isNotEmpty(delBusinessTimes)) {
            for (Integer delId : delBusinessTimes) {
                tabStoreBusinessTimeMapper.deleteByPrimaryKey(delId);
            }
        }
        if (CollectionUtils.isNotEmpty(businessTimeList)) {
            for (StoreBusinessTimeDTO storeBusinessTimeDTO : businessTimeList) {
                if (StringUtils.isNotBlank(storeBusinessTimeDTO.getWeekday())) {
                    TabStoreBusinessTime businessTime = new TabStoreBusinessTime();
                    businessTime.setStoreBusinessTimeId(storeBusinessTimeDTO.getStoreBusinessTimeId());
                    if (StringUtils.isNotBlank(storeBusinessTimeDTO.getWeekday())) {
                        String weekday = Stream.of(storeBusinessTimeDTO.getWeekday().split(GlobalInfo.FLAG_COMMA)).sorted().reduce((x, y) -> x + GlobalInfo.FLAG_COMMA + y).get();
                        businessTime.setWeekday(weekday);
                    }
                    businessTime.setOpenTime(storeBusinessTimeDTO.getOpenTime());
                    businessTime.setCloseTime(storeBusinessTimeDTO.getCloseTime());
                    businessTime.setEnterpriseId(enterpriseId);
                    businessTime.setStoreInfoId(storeInfoId);
                    if (storeBusinessTimeDTO.getStoreBusinessTimeId() == null) {
                        businessTime.setCreateTime(new Date());
                        tabStoreBusinessTimeMapper.insertSelective(businessTime);
                    } else {
                        tabStoreBusinessTimeMapper.updateByPrimaryKeySelective(businessTime);
                    }
                }
            }
        }
    }

    @Override
    public List<TabStoreBusinessTime> listBusinessTime(Integer storeId) {
        return tabStoreBusinessTimeMapper.listBusinessTime(storeId);
    }

    @Override
    public void convertBusinessTime(List<StoreBusinessTimeDTO> businessTimeDTOList) {
        for (StoreBusinessTimeDTO storeBusinessTimeDTO : businessTimeDTOList) {
            String weekday = storeBusinessTimeDTO.getWeekday();
            if (StringUtils.isNotBlank(weekday)) {
                List<Integer> list = Stream.of(weekday.split(GlobalInfo.FLAG_COMMA)).filter(e -> StringUtils.isNotBlank(e)).map(Integer::parseInt).collect(Collectors.toList());
                if (list.size() >= 3 && list.get(list.size() - 1) - list.get(0) == list.size() - 1) {
                    // 超过三条的需要判断
                    // 都是连续的
                    storeBusinessTimeDTO.setWeekdayShow(WeekdayEnum.parse(list.get(0)) + "至" + WeekdayEnum.parse(list.get(list.size() - 1)));
                } else {
                    // 直接解析返回
                    StringBuilder sb = new StringBuilder();
                    for (Integer code : list) {
                        sb.append(WeekdayEnum.parse(code)).append("、");
                    }
                    sb.deleteCharAt(sb.length() - 1);
                    storeBusinessTimeDTO.setWeekdayShow(sb.toString());
                }
            }
        }
    }
}
