package com.gic.store.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.CollectionUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.store.constant.StoreLogTypeEnum;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.*;
import com.gic.store.entity.TabClerk;
import com.gic.store.entity.TabStoreGroup;
import com.gic.store.service.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @date 2019/7/16
 */
@Service("clerkApiService")
public class ClerkApiServiceImpl implements ClerkApiService {

    @Autowired
    private ClerkService clerkService;
    @Autowired
    private StoreGroupService storeGroupService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private ClerkLogService clerkLogService;
    @Autowired
    private StoreLogService storeLogService;

    @Override
    public ServiceResponse saveOrUpdate(ClerkDTO clerkDTO) {
        if(validStoreIsOther(clerkDTO.getEnterpriseId(), clerkDTO.getStoreInfoId())){
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "权限不足");
        }
        // 校验参数
        if (clerkService.isRepeatByClerkCode(clerkDTO.getStoreInfoId(), clerkDTO.getClerkCode(), clerkDTO.getClerkId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购关联主键重复");
        }
        if (clerkService.isRepeatByPhoneNumber(clerkDTO.getStoreInfoId(), clerkDTO.getPhoneNumber(), clerkDTO.getClerkId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号重复");
        }
        if (clerkService.isRepeatByClerkName(clerkDTO.getStoreInfoId(), clerkDTO.getClerkName(), clerkDTO.getClerkId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购名称重复");
        }
        if (clerkDTO.getClerkId() == null) {
//            Integer saveClerkLogBySaveClerk(Integer reason, Integer enterpriseId, Integer clerkId, String remark, Integer operatorId, String operatorName);
            Integer id = clerkService.save(clerkDTO);
            clerkLogService.saveClerkLogBySaveClerk(clerkDTO.getEnterpriseId(), id, clerkDTO.getReason(), clerkDTO.getRemark(), clerkDTO.getOperatorId(), clerkDTO.getOperatorName());
            storeLogByAddClerk(clerkDTO);
        } else {
            TabClerk oldClerk = this.clerkService.getById(clerkDTO.getClerkId());
            clerkService.update(clerkDTO);
            clerkLogService.saveClerkLogByUpdateClerk(oldClerk, clerkDTO, clerkDTO.getEnterpriseId(), clerkDTO.getClerkId(), clerkDTO.getReason(), clerkDTO.getRemark(), clerkDTO.getOperatorId(), clerkDTO.getOperatorName());
        }
        return ServiceResponse.success();
    }

    private void storeLogByAddClerk(ClerkDTO clerkDTO) {
        StoreLogDTO storeLogDTO = new StoreLogDTO();
        storeLogDTO.setEnterpriseId(clerkDTO.getEnterpriseId());
        Integer storeId = this.storeService.getStoreIdByStoreInfoId(clerkDTO.getEnterpriseId(), clerkDTO.getStoreInfoId());
        storeLogDTO.setStoreId(storeId);
        storeLogDTO.setContent("门店关联导购【导购编码" + clerkDTO.getClerkCode() + "】");
        storeLogDTO.setReason(clerkDTO.getReason());
        storeLogDTO.setLogType(StoreLogTypeEnum.ADD_CLERK.getCode());
        storeLogDTO.setRemark(clerkDTO.getRemark());
        storeLogDTO.setOperatorId(clerkDTO.getOperatorId());
        storeLogDTO.setOperatorName(clerkDTO.getOperatorName());

        storeLogService.saveStoreLog(storeLogDTO);
    }

    private boolean validStoreIsOther(Integer enterpriseId, Integer storeInfoId) {
        Integer owner = storeService.getStoreOwnerByStoreInfoId(enterpriseId, storeInfoId);
        return owner == StoreOwnTypeEnum.OTHER.getCode();
    }

    @Override
    @SuppressWarnings("unchecked")
    public ServiceResponse<Page<ClerkStoreListDTO>> listClerk(Integer enterpriseId, ClerkSearchDTO clerkSearchDTO, Integer pageNum, Integer pageSize){
        if (clerkSearchDTO.getStoreGroupId() == null) {
            return ServiceResponse.failure(ErrorCode.MISS_PARAMETER.getCode(), ErrorCode.MISS_PARAMETER.getMsg());
        }
        // 分组的id
        List<TabStoreGroup> groupList = storeGroupService.listStoreGroupAndChildren(clerkSearchDTO.getStoreGroupId());
        List<Integer> groupIds = groupList.stream().map(TabStoreGroup::getStoreGroupId).collect(Collectors.toList());
//         查询条件对应的storeId
        String storeIdByClerkSearch;
        if (StringUtils.isNotBlank(clerkSearchDTO.getSearch())) {
            List<Integer> storeInfoIdsBySearch = clerkService.getStoreInfoIdsBySearch(enterpriseId, clerkSearchDTO.getSearch());
            if (CollectionUtils.isEmpty(groupIds) || CollectionUtils.isEmpty(storeInfoIdsBySearch)) {
                return ServiceResponse.success(new Page<>());
            }
            storeIdByClerkSearch = getStoreIdByStoreInfoIds(storeInfoIdsBySearch);
        } else {
            storeIdByClerkSearch = "";
        }
        clerkSearchDTO.setEnterpriseId(enterpriseId);
        clerkSearchDTO.setStoreIds(storeIdByClerkSearch);
        ServiceResponse<Page<StoreDTO>> serviceResponse = listStoreIds(clerkSearchDTO, pageNum, pageSize);
        if (!serviceResponse.isSuccess()) {
            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        Page<StoreDTO> page = serviceResponse.getResult();
        List<StoreDTO> storeList = page.getResult();
        List<Integer> storeInfoIds = new ArrayList<>(storeList.size());
        List<ClerkStoreListDTO> storeClerkDTOList = new ArrayList<>(storeList.size());
        for (StoreDTO store : storeList) {
            ClerkStoreListDTO clerkStoreListDTO = new ClerkStoreListDTO();
            clerkStoreListDTO.setStoreId(store.getStoreId());
            clerkStoreListDTO.setStoreInfoId(store.getStoreInfoId());
            clerkStoreListDTO.setStoreName(store.getStoreName());
            clerkStoreListDTO.setStoreCode(store.getStoreCode());
            clerkStoreListDTO.setOwnType(store.getOwnType());
            storeClerkDTOList.add(clerkStoreListDTO);
            storeInfoIds.add(store.getStoreInfoId());
        }
        List<TabClerk> clerkList = clerkService.listClerkByStoreInfoId(enterpriseId, storeInfoIds, clerkSearchDTO.getSearch());
        Map<Integer, List<TabClerk>> storeIdMap = CollectionUtil.group(clerkList, "storeInfoId");
        for (ClerkStoreListDTO clerkStoreListDTO : storeClerkDTOList) {
            List<TabClerk> tabClerks = storeIdMap.get(clerkStoreListDTO.getStoreInfoId());
            if (CollectionUtils.isNotEmpty(tabClerks)) {
                clerkStoreListDTO.setClerkList(EntityUtil.changeEntityListByOrika(ClerkDTO.class, tabClerks));
            } else {
                clerkStoreListDTO.setClerkList(Collections.emptyList());
            }
        }
        Page resultPage = new Page();
        resultPage.setPageSize(page.getPageSize());
        resultPage.setCurrentPage(page.getCurrentPage());
        resultPage.setTotalCount(page.getTotalCount());
        resultPage.setTotalPage(page.getPages());
        resultPage.setResult(storeClerkDTOList);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Integer> getTotalClerk(Integer enterpriseId, ClerkSearchDTO clerkSearchDTO) {
        if (clerkSearchDTO.getStoreGroupId() == null) {
            return ServiceResponse.failure(ErrorCode.MISS_PARAMETER.getCode(), ErrorCode.MISS_PARAMETER.getMsg());
        }
        // 分组的id
        List<TabStoreGroup> groupList = storeGroupService.listStoreGroupAndChildren(clerkSearchDTO.getStoreGroupId());
        List<Integer> groupIds = groupList.stream().map(TabStoreGroup::getStoreGroupId).collect(Collectors.toList());
        //         查询条件对应的storeId
        String storeIdByClerkSearch;
        if (StringUtils.isNotBlank(clerkSearchDTO.getSearch())) {
            List<Integer> storeInfoIdsBySearch = clerkService.getStoreInfoIdsBySearch(enterpriseId, clerkSearchDTO.getSearch());
            if (CollectionUtils.isEmpty(groupIds) || CollectionUtils.isEmpty(storeInfoIdsBySearch)) {
                return ServiceResponse.success();
            }
            storeIdByClerkSearch = getStoreIdByStoreInfoIds(storeInfoIdsBySearch);
        } else {
            storeIdByClerkSearch = "";
        }
        clerkSearchDTO.setEnterpriseId(enterpriseId);
        clerkSearchDTO.setStoreIds(storeIdByClerkSearch);
        ServiceResponse<Page<StoreDTO>> serviceResponse = listStoreIds(clerkSearchDTO, 1, Integer.MAX_VALUE);
        if (!serviceResponse.isSuccess()) {
            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        Page<StoreDTO> page = serviceResponse.getResult();
        List<Integer> storeInfoIds = page.getResult().stream().map(StoreDTO::getStoreInfoId).collect(Collectors.toList());
        Integer totalClerk = clerkService.getTotalClerk(enterpriseId, storeInfoIds, clerkSearchDTO.getSearch());

        return ServiceResponse.success(totalClerk);
    }

    private String getStoreIdByStoreInfoIds(List<Integer> storeInfoIdsBySearch) {
        StoreSearchDBDTO storeSearchDBDTO = new StoreSearchDBDTO();
        storeSearchDBDTO.setStoreInfoIdList(storeInfoIdsBySearch);
        List<StoreDTO> dtos = storeService.listStore(storeSearchDBDTO, 1, Integer.MAX_VALUE);
        return Optional.ofNullable(dtos)
                .orElse(Collections.emptyList())
                .stream()
                .map(e -> e.getRegionId().toString())
                .reduce((x, y) -> x + " " + y)
                .orElse("");
    }

    @SuppressWarnings("unchecked")
    private ServiceResponse<Page<StoreDTO>> listStoreIds(ClerkSearchDTO clerkSearchDTO, Integer pageNum, Integer pageSize) {
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(clerkSearchDTO.getEnterpriseId());
        storeSearchDTO.setStoreGroupId(clerkSearchDTO.getStoreGroupId());
        storeSearchDTO.setSearchJson(clerkSearchDTO.getSelectJson());
        storeSearchDTO.setStoreIds(clerkSearchDTO.getStoreIds());

        return this.storeApiService.listStore(storeSearchDTO, pageNum, pageSize, "storeId,storeName,storeCode,storeInfoId,ownType");
    }

    @Override
    public ServiceResponse updateClerkStatus(Integer enterpriseId, String clerkIds, Integer status) {
        return ServiceResponse.success(clerkService.updateClerkStatus(enterpriseId, clerkIds, status));
    }

    @Override
    public ServiceResponse<List<ClerkDTO>> listClerkByIds(Integer enterpriseId, String clerkIds) {
        List<TabClerk> clerkList = clerkService.listClerkByIds(enterpriseId, clerkIds);
        List<ClerkDTO> clerkDTOList = EntityUtil.changeEntityListByJSON(ClerkDTO.class, clerkList);
        return EnterpriseServiceResponse.success(clerkDTOList);
    }

    @Override
    public ServiceResponse<List<ClerkDTO>> listClerkByStoreId(Integer enterpriseId, Integer storeId, String search) {
        ServiceResponse<StoreDTO> response = this.storeApiService.getStoreById(enterpriseId, storeId);
        if(response.isSuccess() && response.getResult() != null){
            List<Integer> storeInfoIds = new ArrayList<>();
            storeInfoIds.add(response.getResult().getStoreInfoId());
            List<TabClerk> clerkList = this.clerkService.listClerkByStoreInfoId(enterpriseId, storeInfoIds, search);
            return ServiceResponse.success(EntityUtil.changeEntityListByJSON(ClerkDTO.class, clerkList));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<ClerkDTO> getClerkById(Integer enterpriseId, Integer clerkId) {
        TabClerk tabClerk = this.clerkService.getById(clerkId);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(ClerkDTO.class, tabClerk));
    }

    @Override
    public ServiceResponse<List<ClerkDTO>> listClerk(Integer enterpriseId, String search) {
        List<TabClerk> clerkList = this.clerkService.listClerk(enterpriseId, search);
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(ClerkDTO.class, clerkList));
    }

}