package com.gic.store.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.bizdict.api.dto.BizdictDTO;
import com.gic.bizdict.api.service.BizdictService;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.dto.StoreDictDTO;
import com.gic.store.dto.StoreFieldDTO;
import com.gic.store.entity.TabStoreDict;
import com.gic.store.entity.TabStoreField;
import com.gic.store.service.StoreDictApiService;
import com.gic.store.service.StoreDictService;
import com.gic.store.service.StoreFieldApiService;
import com.gic.store.service.StoreFieldService;
import com.gic.store.utils.StoreRedisKeyUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service("storeDictApiService")
public class StoreDictApiServiceImpl implements StoreDictApiService {
    @Autowired
    private StoreDictService storeDictService;
    @Autowired
    private BizdictService bizdictService;
    @Autowired
    private StoreFieldService storeFieldService;
    @Autowired
    private StoreFieldApiService storeFieldApiService;

    @Override
    public ServiceResponse<List<StoreDictDTO>> listAllBusinessCategory(Integer enterpriseId) {
        List<StoreDictDTO> result = new ArrayList<>();
        List<BizdictDTO> list = this.bizdictService.treeByCategoryCode("businessCategory");
        if(CollectionUtils.isNotEmpty(list)){
            result = changeListToTree(list, enterpriseId);
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<StoreDictDTO> getStoreDict(String categoryCode, String code) {
        BizdictDTO dto = this.bizdictService.getByCode(categoryCode, code);
        StoreDictDTO storeDictDTO = new StoreDictDTO();
        storeDictDTO.setKey(dto.getName());
        storeDictDTO.setValue(dto.getValueContent());
        return EnterpriseServiceResponse.success(storeDictDTO);
    }

    private List<StoreDictDTO> changeListToTree(List<BizdictDTO> list, Integer enterpriseId) {
        List<StoreDictDTO> result = new ArrayList<>();
        for(BizdictDTO bizdictDTO : list){
            StoreDictDTO dictDTO = new StoreDictDTO();
            dictDTO.setEnterpriseId(enterpriseId);
            dictDTO.setKey(bizdictDTO.getName());
            dictDTO.setValue(bizdictDTO.getValueContent());
            if (CollectionUtils.isNotEmpty(bizdictDTO.getNodeChildren())) {
                List nodeChildren = bizdictDTO.getNodeChildren();
                List<StoreDictDTO> dtoList = changeListToTree((List<BizdictDTO>) nodeChildren, enterpriseId);
                dictDTO.setStoreDictList(dtoList);
            }
            result.add(dictDTO);
        }
        return result;
    }

    @Override
    public ServiceResponse<List<StoreDictDTO>> listAllStoreType(Integer enterpriseId) {
        return ServiceResponse.success(this.listAllStoreDict(enterpriseId, "storeType"));
    }

    @Override
    public ServiceResponse<List<StoreDictDTO>> listAllStoreStatus(Integer enterpriseId) {
        return ServiceResponse.success(this.listAllStoreDict(enterpriseId, "storeStatus"));
    }

    @Override
    public ServiceResponse<List<StoreDictDTO>> listStoreType(Integer enterpriseId) {
        return ServiceResponse.success(this.listStoreDict(enterpriseId, "storeType"));
    }

    @Override
    public ServiceResponse<List<StoreDictDTO>> listStoreStatus(Integer enterpriseId) {
        return ServiceResponse.success(this.listStoreDict(enterpriseId, "storeStatus"));
    }

    @Override
    public ServiceResponse saveStoreType(Integer enterpriseId, String[] storeTypeArr) {
        if(enterpriseId == null || storeTypeArr == null || (storeTypeArr != null && storeTypeArr.length==0)){
            return ServiceResponse.failure(ErrorCode.MISS_PARAMETER.getCode(), ErrorCode.MISS_PARAMETER.getMsg());
        }
        List<String> listSelected = this.storeDictService.listStoreType(enterpriseId);
        for(String storeType : storeTypeArr){
            boolean isExist = false;
            for(String s : listSelected){
                if(s.equals(storeType)){
                    isExist = true;
                }
            }
            if(!isExist && StringUtils.isNotBlank(storeType)){
                TabStoreDict tabStoreDict = new TabStoreDict();
                tabStoreDict.setEnterpriseId(enterpriseId);
                tabStoreDict.setCreateTime(new Date());
                tabStoreDict.setValue(storeType);
                this.storeDictService.saveStoreType(tabStoreDict);
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse saveStoreStatus(Integer enterpriseId, String[] storeStatusArr) {
        if(enterpriseId == null || storeStatusArr == null || (storeStatusArr != null && storeStatusArr.length==0)){
            return ServiceResponse.failure(ErrorCode.MISS_PARAMETER.getCode(), ErrorCode.MISS_PARAMETER.getMsg());
        }
        List<String> listSelected = this.storeDictService.listStoreStatus(enterpriseId);
        for(String storeStatus : storeStatusArr){
            boolean isExist = false;
            for(String s : listSelected){
                if(s.equals(storeStatus)){
                    isExist = true;
                }
            }
            if(!isExist && StringUtils.isNotBlank(storeStatus)){
                TabStoreDict tabStoreDict = new TabStoreDict();
                tabStoreDict.setEnterpriseId(enterpriseId);
                tabStoreDict.setCreateTime(new Date());
                tabStoreDict.setValue(storeStatus);
                this.storeDictService.saveStoreStatus(tabStoreDict);
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse saveStoreStatusAndStoreType(Integer enterpriseId, String[] storeStatusArr, String[] storeTypeArr) {
        this.saveStoreType(enterpriseId, storeTypeArr);
        this.saveStoreStatus(enterpriseId, storeStatusArr);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<StoreDictDTO>> listStoreField(Integer enterpriseId) {
        List<StoreDictDTO> result = new ArrayList<>();
        ServiceResponse<List<StoreFieldDTO>> listStoreField = storeFieldApiService.listStoreField(enterpriseId, null);
        if(listStoreField.isSuccess()){
            List<StoreFieldDTO> list = listStoreField.getResult();
            List<String> listSelect = this.storeDictService.listStoreField(enterpriseId);
            for(StoreFieldDTO tabStoreField : list){
                StoreDictDTO storeDictDTO = new StoreDictDTO();
                storeDictDTO.setKey(tabStoreField.getStoreFieldName());
                storeDictDTO.setValue(tabStoreField.getStoreFieldId()+"");
                storeDictDTO.setOwnType(tabStoreField.getOwnType());
                for(String s : listSelect){
                    if(tabStoreField.getStoreFieldId().intValue() == Integer.valueOf(s).intValue()){
                        storeDictDTO.setChecked(1);
                    }
                }
                result.add(storeDictDTO);
            }
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse saveStoreField(Integer enterpriseId, String value, int type, Integer userId) {
        String key = StoreRedisKeyUtils.getStoreFieldKey(enterpriseId, Integer.valueOf(value));
        String keyc = StoreRedisKeyUtils.getStoreFieldKeyC(enterpriseId, "");
        if(type == 1){ //新增
            TabStoreField tabStoreField = this.storeFieldService.getStoreFieldById(Integer.valueOf(value));
            if(tabStoreField.getStoreFieldType() != null && tabStoreField.getStoreFieldType().intValue() == 1){
                return EnterpriseServiceResponse.failure(ErrorCode.NOT_SUPPORT);
            }
            List<String> list = this.storeDictService.listStoreField(enterpriseId);
            for(String s : list){
                if(s.equals(value)){
                    return ServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), ErrorCode.DATA_EXISTS.getMsg());
                }
            }
            TabStoreDict tabStoreDict = new TabStoreDict();
            tabStoreDict.setEnterpriseId(enterpriseId);
            tabStoreDict.setCreateTime(new Date());
            tabStoreDict.setValue(value);
            this.storeDictService.saveStoreField(tabStoreDict);

            Object existObj = RedisUtil.getCache(key);
            if(existObj == null){
                for(int i=1; i<=10; i++){
                    Object obj = RedisUtil.getCache(keyc+"c"+i);
                    if(obj == null){
                        RedisUtil.setCache(keyc+"c"+i, value);
                        RedisUtil.setCache(key, "c"+i);
                        break;
                    }
                }
            }
        }else {
            this.storeDictService.deleteStoreField(enterpriseId, value);
            Object c = RedisUtil.getCache(key);
            String storeListSourceKey = StoreRedisKeyUtils.getStoreListSourceKey(enterpriseId, userId);
            if(c != null){
                RedisUtil.delCache(c.toString());
                //门店列表自定义字段配置，取消时需要删除当前已经配置的列表显示字段
                Object storeListSource = RedisUtil.getCache(storeListSourceKey);
                if(storeListSource != null){
                    String[] arr = storeListSource.toString().split(",");
                    String result = "";
                    for(String a : arr){
                        if(!c.toString().equals(a)){
                            result += a+",";
                        }
                    }
                    if(StringUtils.isNotBlank(result)){
                        RedisUtil.setCache(storeListSourceKey, result);
                    }
                }
            }
            RedisUtil.delCache(key);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<String>> listStoreFieldDictValue(Integer enterpriseId) {
        return ServiceResponse.success(this.storeDictService.listStoreField(enterpriseId));
    }

    private List<StoreDictDTO> listAllStoreDict(Integer enterpriseId, String type){
        List<StoreDictDTO> result = new ArrayList<>();
        List<BizdictDTO> list = this.bizdictService.treeByCategoryCode(type);
        if(CollectionUtils.isNotEmpty(list)){
            List<String> listSelected = new ArrayList<>();
            if("storeType".equals(type)){
                listSelected = this.storeDictService.listStoreType(enterpriseId);
            }else{
                listSelected = this.storeDictService.listStoreStatus(enterpriseId);
            }
            for(BizdictDTO bizdictDTO : list){
                StoreDictDTO dictDTO = new StoreDictDTO();
                dictDTO.setEnterpriseId(enterpriseId);
                dictDTO.setKey(bizdictDTO.getName());
                dictDTO.setValue(bizdictDTO.getValueContent());
                for(String s : listSelected){
                    if(s.equals(bizdictDTO.getValueContent())){
                        dictDTO.setChecked(1);
                    }
                }
                result.add(dictDTO);
            }
        }
        return result;
    }

    private List<StoreDictDTO> listStoreDict(Integer enterpriseId, String type){
        List<StoreDictDTO> result = new ArrayList<>();
        List<BizdictDTO> list = this.bizdictService.treeByCategoryCode(type);
        if(CollectionUtils.isNotEmpty(list)){
            List<String> listSelected = new ArrayList<>();
            if("storeType".equals(type)){
                listSelected = this.storeDictService.listStoreType(enterpriseId);
            }else{
                listSelected = this.storeDictService.listStoreStatus(enterpriseId);
            }
            for(BizdictDTO bizdictDTO : list){
                for(String s : listSelected){
                    if(s.equals(bizdictDTO.getValueContent())){
                        StoreDictDTO dictDTO = new StoreDictDTO();
                        dictDTO.setEnterpriseId(enterpriseId);
                        dictDTO.setKey(bizdictDTO.getName());
                        dictDTO.setValue(bizdictDTO.getValueContent());
                        dictDTO.setChecked(1);
                        result.add(dictDTO);
                    }
                }
            }
        }
        return result;
    }
}
