package com.gic.store.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.SignConstants;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import com.gic.store.constant.StoreFieldTypeEnum;
import com.gic.store.dto.StoreFieldSelectDTO;
import com.gic.store.entity.TabStoreField;
import com.gic.store.entity.TabStoreFieldSelect;
import com.gic.store.service.StoreExtendService;
import com.gic.store.service.StoreFieldSelectApiService;
import com.gic.store.service.StoreFieldSelectService;
import com.gic.store.service.StoreFieldService;
import com.gic.store.utils.valid.ValidUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author guojx
 * @date 2019/7/1 2:52 PM
 */
@Service("storeFieldSelectApiService")
public class StoreFieldSelectApiServiceImpl implements StoreFieldSelectApiService {
    @Autowired
    private StoreFieldSelectService storeFieldSelectService;
    @Autowired
    private StoreExtendService storeExtendService;
    @Autowired
    private StoreFieldService storeFieldService;

    @Override
    public ServiceResponse<Integer> save(StoreFieldSelectDTO storeFieldSelectDTO) {
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(storeFieldSelectDTO,
                StoreFieldSelectDTO.SaveStoreFieldSelectValid.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }

        if (storeFieldSelectService.isRepeatCode(storeFieldSelectDTO.getStoreFieldId(), null,
                storeFieldSelectDTO.getStoreFieldSelectCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "属性值code不能重复");
        }

        if (storeFieldSelectService.isRepeatName(storeFieldSelectDTO.getStoreFieldId(), null,
                storeFieldSelectDTO.getStoreFieldSelectName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "属性值名称不能重复");
        }
        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldSelectDTO.getStoreFieldId());
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "属性参数有误");
        }
        int storeFieldType = storeField.getStoreFieldType().intValue();
        if (storeFieldType != StoreFieldTypeEnum.RADIO.getCode()
                && storeFieldType != StoreFieldTypeEnum.CHECK.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "属性类型有误");
        }

        storeFieldSelectDTO.setCreateTime(new Date());
        storeFieldSelectDTO.setUpdateTime(new Date());
        storeFieldSelectDTO.setSort(storeFieldSelectService.getMinSort(storeFieldSelectDTO.getStoreFieldId()) - 1);

        return ServiceResponse.success(storeFieldSelectService.save(storeFieldSelectDTO));
    }

    @Override
    public ServiceResponse<Integer> upOrDown(int storeFieldSelectId, boolean isUp) {
        TabStoreFieldSelect tabStoreFieldSelect = storeFieldSelectService.getStoreFieldSelectById(storeFieldSelectId);
        if (tabStoreFieldSelect == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }
        TabStoreFieldSelect needUpdateSort;
        if (isUp) {
            needUpdateSort = storeFieldSelectService.getPrevSort(tabStoreFieldSelect.getStoreFieldId(),
                    tabStoreFieldSelect.getSort());
            if (needUpdateSort == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已经是第一条，不能上移");
            }
        } else {
            needUpdateSort = storeFieldSelectService.getNextSort(tabStoreFieldSelect.getStoreFieldId(),
                    tabStoreFieldSelect.getSort());
            if (needUpdateSort == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已经是最后一条，不能下移");
            }
        }
        storeFieldSelectService.updateSortById(needUpdateSort.getStoreFieldSelectId(), tabStoreFieldSelect.getSort());
        storeFieldSelectService.updateSortById(tabStoreFieldSelect.getStoreFieldSelectId(), needUpdateSort.getSort());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> setStoreFieldSelectSort(int storeFieldSelectId, int setValue) {
        TabStoreFieldSelect tabStoreFieldSelect = storeFieldSelectService.getStoreFieldSelectById(storeFieldSelectId);
        if (tabStoreFieldSelect == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }
        storeFieldSelectService.setStoreFieldSelectSort(storeFieldSelectId, tabStoreFieldSelect.getStoreFieldId(),
                setValue);
        return ServiceResponse.success(tabStoreFieldSelect.getStoreFieldSelectName());
    }

    @Override
    public ServiceResponse<String> delete(String storeFieldSelectIds) {
        String[] storeFieldSelectArr = storeFieldSelectIds.split(",");
        ServiceResponse validResult = validDelete(storeFieldSelectArr);
        if (!validResult.isSuccess()) {
            return validResult;
        }
        storeFieldSelectService.deleteBatch(transferStringToInteger(storeFieldSelectArr));

        List<TabStoreFieldSelect> selectList = storeFieldSelectService
                .listStoreFieldSelectByIds(transferStringToInteger(storeFieldSelectArr));
        StringBuilder string = new StringBuilder();
        if (CollectionUtils.isNotEmpty(selectList)) {
            for (TabStoreFieldSelect select : selectList) {
                string.append(select.getStoreFieldSelectName()).append(SignConstants.COMMA);
            }
        }
        return ServiceResponse.success(string.toString());
    }

    @Override
    public ServiceResponse<Integer> edit(StoreFieldSelectDTO storeFieldSelectDTO) {
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(storeFieldSelectDTO,
                StoreFieldSelectDTO.EditStoreFieldSelectValid.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }

        if (storeFieldSelectService.isRepeatCode(storeFieldSelectDTO.getStoreFieldId(),
                storeFieldSelectDTO.getStoreFieldSelectId(), storeFieldSelectDTO.getStoreFieldSelectCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "属性值code不能重复");
        }

        if (storeFieldSelectService.isRepeatName(storeFieldSelectDTO.getStoreFieldId(),
                storeFieldSelectDTO.getStoreFieldSelectId(), storeFieldSelectDTO.getStoreFieldSelectName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "属性值名称不能重复");
        }

        return ServiceResponse.success(storeFieldSelectService.edit(storeFieldSelectDTO.getStoreFieldSelectId(),
                storeFieldSelectDTO.getStoreFieldSelectName(), storeFieldSelectDTO.getStoreFieldSelectCode()));
    }

    @Override
    public ServiceResponse<Page<StoreFieldSelectDTO>> listStoreFieldSelect(int storeFieldId, String search,
            Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page listStoreFieldSelect = storeFieldSelectService.listStoreFieldSelect(storeFieldId,
                search, pageNum, pageSize);
        Page<StoreFieldSelectDTO> selectDTOPage = PageHelperUtils.changePageHelperToCurrentPage(listStoreFieldSelect,
                StoreFieldSelectDTO.class);
        List<StoreFieldSelectDTO> selectDTOList = selectDTOPage.getResult();
        int index = (pageNum - 1) * pageSize + 1;
        for (int i = 0, length = selectDTOList.size(); i < length; i++) {
            selectDTOList.get(i).setSeq(index + i);
        }
        return ServiceResponse.success(selectDTOPage);
    }

    @Override
    public ServiceResponse<List<StoreFieldSelectDTO>> listStoreFieldSelect(int storeFieldId, String search) {
        return ServiceResponse.success(EntityUtil.changeEntityListNew(StoreFieldSelectDTO.class,
                storeFieldSelectService.listStoreFieldSelect(storeFieldId, search)));
    }

    @Override
    public ServiceResponse<StoreFieldSelectDTO> getStoreFieldSelect(int storeFieldSelectId) {
        return ServiceResponse.success(EntityUtil.changeEntityNew(StoreFieldSelectDTO.class,
                storeFieldSelectService.getStoreFieldSelectById(storeFieldSelectId)));
    }

    @Override
    public ServiceResponse<Integer> sortBottom(int storeFieldSelectId) {
        TabStoreFieldSelect tabStoreFieldSelect = storeFieldSelectService.getStoreFieldSelectById(storeFieldSelectId);
        if (tabStoreFieldSelect == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }
        storeFieldSelectService.sortBottom(tabStoreFieldSelect.getStoreFieldId(), storeFieldSelectId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> sortTopping(int storeFieldSelectId) {
        TabStoreFieldSelect tabStoreFieldSelect = storeFieldSelectService.getStoreFieldSelectById(storeFieldSelectId);
        if (tabStoreFieldSelect == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
        }
        storeFieldSelectService.sortTopping(tabStoreFieldSelect.getStoreFieldId(), storeFieldSelectId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<StoreFieldSelectDTO>> listStoreFieldSelectByIds(Integer enterpriseId,
            List<Integer> storeFileIdSelectIdList) {
        List<TabStoreFieldSelect> list = this.storeFieldSelectService
                .listStoreFieldSelectByIds(storeFileIdSelectIdList);
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(StoreFieldSelectDTO.class, list));
    }

    private List<Integer> transferStringToInteger(String[] storeFieldSelectArr) {
        List<Integer> list = new ArrayList<>(storeFieldSelectArr.length);
        for (String str : storeFieldSelectArr) {
            list.add(Integer.parseInt(str));
        }
        return list;
    }

    private ServiceResponse validDelete(String[] storeFieldSelectArr) {
        for (String select : storeFieldSelectArr) {
            TabStoreFieldSelect tabStoreFieldSelect = storeFieldSelectService
                    .getStoreFieldSelectById(Integer.parseInt(select));
            if (tabStoreFieldSelect == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键错误，查询不到数据");
            }
            if (storeExtendService.storeExtendHasStoreFieldSelect(tabStoreFieldSelect.getStoreFieldId(),
                    Integer.parseInt(select))) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                        "属性值 " + tabStoreFieldSelect.getStoreFieldSelectName() + "在使用中，不允许删除");
            }
        }
        return ServiceResponse.success();
    }

}
