package com.gic.store.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.store.constant.StoreFieldTypeEnum;
import com.gic.store.dto.StoreExtendDTO;
import com.gic.store.dto.StoreFieldDTO;
import com.gic.store.dto.StoreTmpDTO;
import com.gic.store.entity.TabStoreBrand;
import com.gic.store.entity.TabStoreExtend;
import com.gic.store.entity.TabStoreField;
import com.gic.store.entity.TabStoreGroup;
import com.gic.store.service.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @date 2019/7/3
 */
@Service("storeImportApiService")
@SuppressWarnings("unchecked")
public class StoreImportApiServiceImpl implements StoreImportApiService {

    private static final Logger logger = LogManager.getLogger(StoreBrandApiServiceImpl.class);

    @Autowired
    private StoreBrandService storeBrandService;

    @Autowired
    private StoreService storeService;

    @Autowired
    private StoreImportService storeImportService;

    @Autowired
    private StoreGroupService storeGroupService;

    @Autowired
    private StoreTaskApiService storeTaskService;

    @Autowired
    private StoreExtendService storeExtendService;

    @Autowired
    private StoreFieldService storeFieldService;

    @Override
    public ServiceResponse<Boolean> hasImportingStore(Integer enterpriseId) {
        return ServiceResponse.success(storeImportService.hasImportingStore(enterpriseId));
    }

    @Override
    public ServiceResponse<Void> delStoreTmp(Integer enterpriseId) {
        storeImportService.delStoreTmp(enterpriseId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> save(StoreTmpDTO bean) {
        storeImportService.save(bean);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page> listStoreTmp(Integer enterpriseId, boolean isSuccess, Integer currentPage, Integer pageSize) {
        com.github.pagehelper.Page storeTmpList = storeImportService.listStoreTmp(enterpriseId, isSuccess, currentPage, pageSize);
        Page<StoreTmpDTO> page = PageHelperUtils.changePageHelperToCurrentPage(storeTmpList, StoreTmpDTO.class);
        List<StoreTmpDTO> result = page.getResult();
        for (StoreTmpDTO storeTmpDTO : result) {
            String customField = storeTmpDTO.getCustomField();
            List<StoreExtendDTO> extendList = storeExtendService.convertStoreExtendToDTO(JSON.parseArray(customField, TabStoreExtend.class));
            storeTmpDTO.setExtendList(extendList);
        }
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<Boolean> validateStoreCode(Integer enterpriseId, Integer regionId, String storeCode) {
        return ServiceResponse.success(storeService.validateStoreCodeIsExist(enterpriseId, regionId, storeCode, null));
    }

    @Override
    public ServiceResponse<String> isExistStoreGroup(Integer enterpriseId, String storeGroupName) {
        TabStoreGroup group = storeGroupService.getStoreGroupByName(storeGroupName, enterpriseId);
        return ServiceResponse.success(group == null ? null : group.getStoreGroupName());
    }

    @Override
    public ServiceResponse<List<String>> listBrandByNotExist(Integer enterpriseId, String brands) {
        if (StringUtils.isBlank(brands)) {
            return ServiceResponse.success();
        }
        List<String> resultList = new ArrayList<>();
        List<TabStoreBrand> storeBrandList = storeBrandService.listAllStoreBrand(enterpriseId, null);
        if (CollectionUtils.isEmpty(storeBrandList)) {
            return ServiceResponse.success();
        }
        List<String> storeBrandNameList = storeBrandList.stream().map(TabStoreBrand::getStoreBrandName).collect(Collectors.toList());
        String[] brandArr = brands.split("、");
        // 这里如果门店品牌很多， 可以改成map 去做
        for (String brandName : brandArr) {
            if (!storeBrandNameList.contains(brandName)) {
                resultList.add(brandName);
            }
        }

        return ServiceResponse.success(resultList);
    }

    @Override
    public ServiceResponse<List<StoreTmpDTO>> listStoreTmp(Integer enterpriseId, Boolean isRight) {
        List<StoreTmpDTO> result = EntityUtil.changeEntityListByOrika(StoreTmpDTO.class, storeImportService.listStoreTmp(enterpriseId, isRight));
        for (StoreTmpDTO storeTmpDTO : result) {
            String customField = storeTmpDTO.getCustomField();
            List<StoreExtendDTO> extendList = storeExtendService.convertStoreExtendToDTO(JSON.parseArray(customField, TabStoreExtend.class));
            storeTmpDTO.setExtendList(extendList);
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<StoreTmpDTO>> listAllStoreTmp(Integer enterpriseId) {
        return ServiceResponse.success(EntityUtil.changeEntityListByOrika(StoreTmpDTO.class, storeImportService.listAllStoreTmp(enterpriseId)));
    }

    @Override
    public ServiceResponse<List<List<String>>> listError(Integer enterpriseId) {
        List<List<String>> resultList = new LinkedList<>();

        List<StoreTmpDTO> dataList = this.listStoreTmp(enterpriseId, Boolean.FALSE).getResult();
        // title
        // 系统title
        // 自定义title
        List<String> title = getTitle(enterpriseId, dataList.get(0).getRegionId());
        resultList.add(title);
        // 数据
        // 普通数据
        // 自定义数据
        packData(resultList, dataList);
        return ServiceResponse.success(resultList);
    }

    private List<String> getTitle(Integer enterpriseId, Integer regionId) {
        List<String> titleList = this.listImportTitle(enterpriseId, regionId).getResult().get(0);
        titleList.add("错误信息");
        return titleList;
    }

    @Override
    public ServiceResponse<Page> pageError(Integer enterpriseId, Integer currentPage, Integer pageSize) {
        List<List<String>> resultList = new LinkedList<>();
        Page page = this.listStoreTmp(enterpriseId, Boolean.FALSE, currentPage, pageSize).getResult();
        List<StoreTmpDTO> dataList = page.getResult();
        if (CollectionUtils.isNotEmpty(dataList)) {
            List<String> title = getTitle(enterpriseId, dataList.get(0).getRegionId());
            resultList.add(title);
            packData(resultList, dataList);
            page.setResult(resultList);
        }
        return ServiceResponse.success(page);
    }

    private void packData(List<List<String>> resultList, List<StoreTmpDTO> dataList) {
        for (int i = 0; i < dataList.size(); i++) {
            List<String> list = new LinkedList<>();
            StoreTmpDTO storeTmpDTO = dataList.get(i);
            //{"序号", "门店名称", "门店代码", "门店类型", "门店状态", "门店电话", "门店所属分组", "关联门店品牌", "省", "市", "区县", "门店地址（不包括省市区）", "经度", "纬度"};
            list.add(i + 1 + "");
            list.add(storeTmpDTO.getStoreName());
            list.add(storeTmpDTO.getStoreCode());
            list.add(storeTmpDTO.getStoreType());
            list.add(storeTmpDTO.getErpStatus());
            list.add(storeTmpDTO.getConactsPhone());
            list.add(storeTmpDTO.getStoreGroupName());
            list.add(storeTmpDTO.getBrands());
            list.add(storeTmpDTO.getProvince());
            list.add(storeTmpDTO.getCity());
            list.add(storeTmpDTO.getCounty());
            list.add(storeTmpDTO.getAddress());
            list.add(storeTmpDTO.getLongitude());
            list.add(storeTmpDTO.getLatitude());

            List<StoreExtendDTO> extendList = storeTmpDTO.getExtendList();
            if (CollectionUtils.isNotEmpty(extendList)) {
                List<String> valueList = extendList.stream().map(StoreExtendDTO::getValue).collect(Collectors.toList());
                list.addAll(valueList);
            }
            list.add(storeTmpDTO.getErrorMessage());

            resultList.add(list);
        }
    }


    @Override
    public ServiceResponse<String> importDataToStore(Integer enterpriseId, String userId, String uuId, boolean isReplace) {
        return ServiceResponse.success(storeTaskService.importDataToStore(enterpriseId, userId, uuId, isReplace));
    }


    @Override
    public ServiceResponse<List<List<String>>> listImportTitle(Integer enterpriseId, Integer regionId) {
        List<List<String>> list = new ArrayList<>();

        String[] rowName = { "序号", "门店名称", "门店代码", "门店类型", "门店状态", "门店电话", "门店所属分组", "关联门店品牌", "省", "市", "区县", "门店地址（不包括省市区）", "经度", "纬度" };
        List<String> rowNameList = new ArrayList<>();
        Collections.addAll(rowNameList, rowName);

        String[] description = { "", "不能超过20个字", "必填，不能超过20个字符，为erp系统中门店唯一识别码", "门店类型描述", "门店状态描述", "门店电话描述", "门店所属分组描述", "用\"、\"隔开", "省描述", "市描述", "区县描述", "门店地址（不包括省市区）描述", "", "" };
        List<String> descriptionList = new ArrayList<>();
        Collections.addAll(descriptionList, description);

        list.add(rowNameList);
        list.add(descriptionList);

        List<StoreFieldDTO> fields = storeFieldService.listStoreFieldByRegionId(regionId);
        if (CollectionUtils.isNotEmpty(fields)) {
            for (StoreFieldDTO field : fields) {
                rowNameList.add(field.getStoreFieldName());
                String storeFieldDetail = field.getStoreFieldDetail();
                JSONObject jsonObject = JSON.parseObject(storeFieldDetail);
                descriptionList.add(jsonObject.getString("tipMessage"));
            }
        }
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<Map<Integer, Boolean>> fieldIsDateType(Integer enterpriseId, Integer regionId) {
        HashMap<Integer, Boolean> map = new HashMap<>();
        List<StoreFieldDTO> fields = storeFieldService.listStoreFieldByRegionId(regionId);
        if (CollectionUtils.isNotEmpty(fields)) {
            for (int i = 0; i < fields.size(); i++) {
                map.put(14 + i, fields.get(i).getStoreFieldType() == StoreFieldTypeEnum.DATE.getCode());
            }
        }
        return ServiceResponse.success(map);
    }

    @Override
    public ServiceResponse<Boolean> validateStoreName(Integer enterpriseId, String storeName) {
        return ServiceResponse.success(storeService.countByStoreName(enterpriseId, storeName, null) > 0);
    }

    @Override
    public ServiceResponse<Map<String, Object>> getStoreBatchProcess(Integer enterpriseId, String uuId, Integer currentPage, Integer pageSize) {
        Map<String, Object> data = new HashMap<>(8);
        Integer taskTotal = storeImportService.countByTaskTotal(enterpriseId, uuId);
        Integer taskExecNum = storeImportService.countByTaskExecNum(enterpriseId, uuId);
        Integer taskFailNum = storeImportService.countByTaskFailNum(enterpriseId, uuId);
        data.put("taskTotal", taskTotal);
        data.put("taskExecNum", taskExecNum);
        data.put("taskFailNum", taskFailNum);
        if ((taskTotal - taskFailNum) > 0) {
            data.put("process", taskExecNum * 100 / (taskTotal - taskFailNum));
            // 获取所有错误数据
            Page resultPage = this.listStoreTmp(enterpriseId, false, currentPage, pageSize).getResult();
            data.put("errorData", resultPage);
        } else {
            data.put("process", 100);
        }
        return ServiceResponse.success(data);
    }

}
