package com.gic.store.service.outer.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreSearchDBDTO;
import com.gic.store.dto.StoreStrategyDTO;
import com.gic.store.entity.TabStoreInfo;
import com.gic.store.entity.TabStoreStrategy;
import com.gic.store.service.StoreService;
import com.gic.store.service.StoreStrategyApiService;
import com.gic.store.service.StoreStrategyService;
import com.gic.store.utils.valid.ValidUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author guojx
 * @date 2019/7/4 10:10 AM
 */
@Service("storeStrategyApiService")
public class StoreStrategyApiServiceImpl implements StoreStrategyApiService{

    @Autowired
    private StoreStrategyService storeStrategyService;
    @Autowired
    private StoreService storeService;

    @Override
    public ServiceResponse<Integer> saveStoreStrategy(StoreStrategyDTO storeStrategyDTO) {
        ServiceResponse validParam = ValidUtil.allCheckValidate(storeStrategyDTO, SaveStoreStrategy.class);
        if (!validParam.isSuccess()) {
            return validParam;
        }

        if (storeStrategyDTO.getStrategyType() == StoreGroupConstant.STORE_STRATEGY_TYPE) {
            //如果是门店状态策略类型，必须有
            if (StringUtils.isBlank(storeStrategyDTO.getStoreCreateType())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店创建方式不能为空");
            }
        }
        String conditions = storeStrategyDTO.getConditions();
        //验证conditions合法性
        if (isConditionsSizeOver(conditions)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店属性条件配置超出5个");
        }
        if (validConditionsIsRepeatKey(conditions)) {
            ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店属性条件每一种都不能重复");
        }

        storeStrategyDTO.setCreateTime(new Date());
        storeStrategyDTO.setUpdateTime(new Date());
        storeStrategyDTO.setStatus(1);
        //sort
        storeStrategyDTO.setSort(storeStrategyService.getMinSort(storeStrategyDTO.getEnterpriseId(), storeStrategyDTO.getStrategyType()) - 1d);

        TabStoreStrategy record = EntityUtil.changeEntityNew(TabStoreStrategy.class, storeStrategyDTO);
        if (storeStrategyService.isRepeatGroupStrategyName(record)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "策略名称不能重复");
        }

        return  ServiceResponse.success(storeStrategyService.saveStoreGroupStrategy(storeStrategyDTO));
    }

    @Override
    public ServiceResponse<String> deleteStoreStrategy(int strategyId) {
        TabStoreStrategy storeStrategy = storeStrategyService.getStoreStrategyById(strategyId);
        if (storeStrategy == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "策略主键ID错误，查询不到数据");
        }
        storeStrategyService.deleteStoreGroupStrategy(strategyId);
        return ServiceResponse.success(storeStrategy.getStrategyName());
    }

    @Override
    public ServiceResponse<Integer> editStoreStrategy(StoreStrategyDTO storeStrategyDTO) {
        ServiceResponse validParam = ValidUtil.allCheckValidate(storeStrategyDTO, SaveStoreStrategy.class);
        if (!validParam.isSuccess()) {
            return validParam;
        }
        TabStoreStrategy storeStrategy = storeStrategyService.getStoreStrategyById(storeStrategyDTO.getStrategyId());
        if (storeStrategy == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "策略主键ID错误，查询不到数据");
        }
        String conditions = storeStrategyDTO.getConditions();
        //验证conditions合法性
        if (isConditionsSizeOver(conditions)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店属性条件配置超出5个");
        }
        if (validConditionsIsRepeatKey(conditions)) {
            ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店属性条件每一种都不能重复");
        }
        if (storeStrategy.getStrategyType() == StoreGroupConstant.STORE_STRATEGY_TYPE) {
            //如果是门店状态策略类型，必须有
            if (StringUtils.isBlank(storeStrategyDTO.getStoreCreateType())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店创建方式不能为空");
            }
        }

        storeStrategyDTO.setCreateTime(storeStrategy.getCreateTime());
        storeStrategyDTO.setUpdateTime(new Date());
        storeStrategyDTO.setEnterpriseId(storeStrategy.getEnterpriseId());
        storeStrategyDTO.setStrategyType(storeStrategy.getStrategyType());

        TabStoreStrategy record = EntityUtil.changeEntityNew(TabStoreStrategy.class, storeStrategyDTO);
        if (storeStrategyService.isRepeatGroupStrategyName(record)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "策略名称不能重复");
        }
        return ServiceResponse.success(storeStrategyService.modifyStoreGroupStrategy(storeStrategyDTO));
    }

    @Override
    public ServiceResponse<StoreStrategyDTO> getStoreStrategyById(int strategyId) {
        TabStoreStrategy storeStrategy = storeStrategyService.getStoreStrategyById(strategyId);
        if (storeStrategy == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "策略主键ID错误，查询不到数据");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(StoreStrategyDTO.class, storeStrategy));
    }

    @Override
    public ServiceResponse<String> sortBottom(int strategyId) {
        TabStoreStrategy storeStrategy = storeStrategyService.getStoreStrategyById(strategyId);
        if (storeStrategy == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "策略主键ID错误，查询不到数据");
        }
        storeStrategyService.sortBottom(storeStrategy.getEnterpriseId(), strategyId, storeStrategy.getStrategyType());
        return ServiceResponse.success(storeStrategy.getStrategyName());
    }

    @Override
    public ServiceResponse<String> sortTopping(int strategyId) {
        TabStoreStrategy storeStrategy = storeStrategyService.getStoreStrategyById(strategyId);
        if (storeStrategy == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "策略主键ID错误，查询不到数据");
        }
        storeStrategyService.sortTopping(storeStrategy.getEnterpriseId(), strategyId, storeStrategy.getStrategyType());
        return ServiceResponse.success(storeStrategy.getStrategyName());
    }

    @Override
    public ServiceResponse<String> upOrDown(int strategyId, boolean isUp) {
        TabStoreStrategy storeStrategy = storeStrategyService.getStoreStrategyById(strategyId);
        if (storeStrategy == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "策略主键ID错误，查询不到数据");
        }
        TabStoreStrategy needUpdateSort;
        if (isUp) {
            needUpdateSort = storeStrategyService.getPrevSort(storeStrategy.getEnterpriseId(), storeStrategy.getSort(), storeStrategy.getStrategyType());
            if (needUpdateSort == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已经是第一条，不能上移");
            }
        } else {
            needUpdateSort = storeStrategyService.getNextSort(storeStrategy.getEnterpriseId(), storeStrategy.getSort(), storeStrategy.getStrategyType());
            if (needUpdateSort == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已经是最后一条，不能下移");
            }
        }
        storeStrategyService.updateSortById(needUpdateSort.getStrategyId(), storeStrategy.getSort());
        storeStrategyService.updateSortById(storeStrategy.getStrategyId(), needUpdateSort.getSort());
        return ServiceResponse.success(storeStrategy.getStrategyName());
    }

    @Override
    public ServiceResponse<String> setStoreStrategySort(int strategyId, int setValue) {
        TabStoreStrategy storeStrategy = storeStrategyService.getStoreStrategyById(strategyId);
        if (storeStrategy == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "策略主键ID错误，查询不到数据");
        }
        storeStrategyService.setStoreStrategySort(strategyId, storeStrategy.getEnterpriseId(), setValue, storeStrategy.getStrategyType());
        return ServiceResponse.success(storeStrategy.getStrategyName());
    }

    @Override
    public ServiceResponse<Page<StoreStrategyDTO>> listStoreField(int enterpriseId, String search, int strategyType, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page storeStrategies = storeStrategyService.listStoreStrategy(enterpriseId, search, strategyType, pageNum, pageSize);
        Page<StoreStrategyDTO> page = PageHelperUtils.changePageHelperToCurrentPage(storeStrategies, StoreStrategyDTO.class);
        List<StoreStrategyDTO> strategyDTOList = page.getResult();
        int index = (pageNum - 1) * pageSize + 1;
        for (int i = 0, length = strategyDTOList.size(); i < length; i++) {
            strategyDTOList.get(i).setSeq(index + i);
        }
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse setStoreGroupStrategyWeight(Integer enterpriseId, int weight) {
        RedisUtil.setCache("enterprise:store:storeGroupStrategy:"+enterpriseId, weight);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse getStoreGroupStrategyWeight(Integer enterpriseId) {
        Object value = RedisUtil.getCache("enterprise:store:storeGroupStrategy:"+enterpriseId);
        if(value == null){
            return ServiceResponse.success(0);
        }
        return ServiceResponse.success(Integer.valueOf(value.toString()));
    }


    @Override
    public ServiceResponse<String> resetStoreGroupStrategy(List<Integer> regionIdList, List<Integer> storeStatusList) {
        StoreSearchDBDTO searchDBDTO = new StoreSearchDBDTO();
        searchDBDTO.setRegionIdList(regionIdList);
        searchDBDTO.setStatusList(storeStatusList);
        com.github.pagehelper.Page<StoreDTO> listStore = this.storeService.listStore(searchDBDTO, 1, Integer.MAX_VALUE);
        for (StoreDTO store : listStore.getResult()) {
            String result = storeStrategyService.isHitStrategy(store.getStoreInfoId(), store.getEnterpriseId(), StoreGroupConstant.STORE_GROUP_STRATEGY_TYPE);
            if (StringUtils.isNotBlank(result)) {
                storeService.updateStoreGroupId(store.getStoreInfoId(), Integer.parseInt(result));
            }
        }
        return ServiceResponse.success();
    }

    private boolean validConditionsIsRepeatKey(String conditions) {
        JSONArray jsonArray = JSONArray.parseArray(conditions);
        //每一种属性都不能一样
        Map<String, String> validConditionsMap = new HashMap<>(16);
        for (int i = 0, length = jsonArray.size(); i < length; i++) {
            JSONObject object = jsonArray.getJSONObject(i);
            if (validConditionsMap.containsKey(object.getString("key"))) {
                return true;
            }
            validConditionsMap.put(object.getString("key"), object.getString("value"));
        }
        return false;
    }

    private boolean isConditionsSizeOver(String conditions) {
        JSONArray jsonArray = JSONArray.parseArray(conditions);
        if (jsonArray.size() > 5) {
            return true;
        }
        return false;
    }
}
