package com.gic.store.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreWidgetApiService;
import com.gic.store.service.StoreWidgetService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service("storeWidgetApiService")
public class StoreWidgetApiServiceImpl implements StoreWidgetApiService {
    @Autowired
    private StoreWidgetService storeWidgetService;
    @Autowired
    private StoreApiService storeApiService;

    @Override
    public ServiceResponse<Integer> saveStoreWidget(StoreWidgetDTO storeWidgetDTO) {
        Integer id = storeWidgetDTO.getStoreWidgetId();
        if(id == null){
            id = this.storeWidgetService.saveStoreWidget(storeWidgetDTO);
        }else {
            this.storeWidgetService.updateStoreWidget(storeWidgetDTO);
        }
        if(id != null && id > 0){
            return ServiceResponse.success(id);
        }
        return EnterpriseServiceResponse.failure(ErrorCode.SYSTEM_ERROR);
    }

    @Override
    public ServiceResponse<Integer> saveAuthStoreWidget(StoreWidgetDTO storeWidgetDTO) {
        Integer id = storeWidgetDTO.getStoreWidgetId();
        if(id == null){
            id = this.storeWidgetService.saveStoreWidget(storeWidgetDTO);
        }else {
            this.storeWidgetService.updateAuthStoreWidget(storeWidgetDTO);
        }
        if(id != null && id > 0){
            return ServiceResponse.success(id);
        }
        return EnterpriseServiceResponse.failure(ErrorCode.SYSTEM_ERROR);
    }

    @Override
    public ServiceResponse<StoreWidgetDTO> getStoreWidget(Integer id) {
        StoreWidgetDTO storeWidgetDTO = this.storeWidgetService.getStoreWidget(id);
        if(storeWidgetDTO != null){
            return ServiceResponse.success(storeWidgetDTO);
        }
        return EnterpriseServiceResponse.failure(ErrorCode.SYSTEM_ERROR);
    }

    @Override
    public ServiceResponse<Integer> updateAuth(StoreWidgetDTO storeWidgetDTO) {
        int i = this.storeWidgetService.updateAuth(storeWidgetDTO);
        if(i > 0){
            return ServiceResponse.success(i);
        }
        return EnterpriseServiceResponse.failure(ErrorCode.SYSTEM_ERROR);
    }

    @Override
    public ServiceResponse<Page<StoreDTO>> listStoreByStoreWidgetId(Integer enterpriseId, Integer storeWidgetId, int pageNum, int pageSize) {
        StoreWidgetDTO storeWidgetDTO = this.storeWidgetService.getStoreWidget(storeWidgetId);
        if(storeWidgetDTO != null){
            String authSearchParam = storeWidgetDTO.getAuthSearchParam();
            String searchParam = storeWidgetDTO.getSearchParam();
            StoreSearchDTO dto = new StoreSearchDTO();
            dto.setEnterpriseId(enterpriseId);
            if(StringUtils.isNotBlank(authSearchParam) && !"{}".equals(authSearchParam)){
                dto.setAuthSearchJson(authSearchParam);
            }
            dto.setSearchJson(searchParam);
            return this.storeApiService.listStore(dto, pageNum, pageSize);
        }
        return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "门店控件id不存在");
    }

    @Override
    public ServiceResponse<List<Integer>> listStoreIdByStoreWidgetId(Integer enterpriseId, Integer storeWidgetId) {
        List<Integer> list = new ArrayList<>();
        this.getStoreId(enterpriseId, storeWidgetId, 1, list);
        return ServiceResponse.success(list);
    }

    private void getStoreId(Integer enterpriseId, Integer storeWidgetId, int pageNum, List<Integer> list){
        ServiceResponse<Page<StoreDTO>> response = this.listStoreByStoreWidgetId(enterpriseId, storeWidgetId, pageNum, 10000);
        if(response.isSuccess() && response.getResult() != null){
            Page<StoreDTO> page = response.getResult();
            for(StoreDTO dto : page.getResult()){
                list.add(dto.getStoreId());
            }
            if(pageNum == 1 && page.getPages() > 1){
                for(int i=2; i<= page.getPages(); i++){
                    this.getStoreId(enterpriseId, storeWidgetId, i, list);
                }
            }
        }
    }
}
