package com.gic.store.utils.log;

import com.gic.store.utils.LogUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 日志aop
 * @ClassName: LogAspect 
 * @Description:  
 * @author guojuxing 
 * @date 2019/7/31 1:52 PM 
 */
@Aspect
public class LogAspect {

    private static final Logger logger = LoggerFactory.getLogger(LogAspect.class);

    @Pointcut("execution(* com.gic.store.service.outer.*.save*(..))")
    public void saveCell() {

    }
    @Pointcut("execution(public * com.gic.store.service.outer.*.update*(..))")
    public void updateCell() {

    }
    @Pointcut("execution(* com.gic.store.service.outer.*.delete*(..))")
    public void deleteCell() {

    }
    @AfterReturning(value = "saveCell()", argNames = "joinPoint,object", returning = "object")
    public void saveLog(JoinPoint joinPoint, Object object) {

        if (joinPoint.getArgs() == null) {
            return;
        }
        String methodName = joinPoint.getSignature().getName();
        String operContent = "";

        logger.info("aspectLog:{}", joinPoint.getArgs());
        LogUtils.saveAddLog(arrToList(joinPoint.getArgs()));
    }
    @AfterReturning(value = "updateCell()", argNames = "joinPoint,object", returning = "object")
    public void updateLog(JoinPoint joinPoint, Object object) {
        if (joinPoint.getArgs() == null) {
            return;
        }
        String methodName = joinPoint.getSignature().getName();
        String operContent = "";

        logger.info("aspectLog:{}", joinPoint.getArgs());
        LogUtils.saveUpdateLog(arrToList(joinPoint.getArgs()));
    }

    private List<Object> arrToList(Object[] args) {
        List<Object> resultList = new ArrayList<>(args.length);
        for (Object obj : args) {
            resultList.add(obj);
        }
        return resultList;
    }
}
