package com.gic.store.web.controller;

import java.util.List;

import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.dto.StoreGroupDTO;
import com.gic.store.service.StoreGroupApiService;
import com.gic.store.utils.CommonResultControllerUtils;
import com.gic.store.utils.storegroup.ChangeListToTree;
import com.gic.store.web.qo.storegroup.StoreGroupQO;

/**
 * 门店分组
 * @ClassName: StoreGroupController 
 * @Description:  
 * @author guojuxing 
 * @date 2019/8/7 10:26 AM 
 */
@RestController
@RequestMapping("/store-group")
public class StoreGroupController {

    private static final Logger logger = LoggerFactory.getLogger(StoreGroupController.class);

    @Autowired
    private StoreGroupApiService storeGroupApiService;

    @RequestMapping("/list")
    public RestResponse listStoreGroup(StoreGroupQO storeGroupQO) {
        ServiceResponse<List<StoreGroupDTO>> result = storeGroupApiService
                .listStoreGroup(transferQoToDTO(storeGroupQO));
        if (result.isSuccess()) {
            return RestResponse
                    .success(ChangeListToTree.changeListToTree(StoreGroupConstant.ALL_STORE_LEVEL, result.getResult()));
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("/save")
    public RestResponse save(@Validated({ StoreGroupQO.SaveValidView.class }) StoreGroupQO storeGroupQO) {
        return OperationResultUtils.operationResult(storeGroupApiService.save(transferQoToDTO(storeGroupQO)),
                OperationResultUtils.LOG_NEW + "门店分组", storeGroupQO.getStoreGroupName());
    }

    @RequestMapping("/edit-store-group-name")
    public RestResponse editName(@Validated({ StoreGroupQO.EditNameValidView.class,
            StoreGroupQO.RemoveValidView.class }) StoreGroupQO storeGroupQO) {
        return OperationResultUtils.operationResult(
                storeGroupApiService.updateStoreGroupName(transferQoToDTO(storeGroupQO)),
                OperationResultUtils.LOG_EDIT + "门店分组", storeGroupQO.getStoreGroupName());
    }

    @RequestMapping("/edit-parent-store-group-id")
    public RestResponse edit(@Validated({ StoreGroupQO.EditParentIdValidView.class,
            StoreGroupQO.RemoveValidView.class }) StoreGroupQO storeGroupQO) {
        ServiceResponse<String> result = storeGroupApiService.update(transferQoToDTO(storeGroupQO));
        return OperationResultUtils.operationResult(result, "修改上级分组", OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/drag-sort")
    public RestResponse dragSort(@Validated({ StoreGroupQO.RemoveValidView.class,
            StoreGroupQO.SetSortValueValidView.class }) StoreGroupQO storeGroupQO) {
        ServiceResponse<String> result = storeGroupApiService.dragSort(storeGroupQO.getStoreGroupId(), storeGroupQO.getSortValue());
        return OperationResultUtils.operationResult(
                result, "拖拽排序门店分组", OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/sort")
    public RestResponse sort(@Validated({ StoreGroupQO.SortValidView.class }) StoreGroupQO storeGroupQO) {
        ServiceResponse<String> result = storeGroupApiService.sort(transferQoToDTO(storeGroupQO), storeGroupQO.isUp());
        return OperationResultUtils.operationResult(
                result, "上下排序门店分组", OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/remove")
    public RestResponse remove(@Validated({ StoreGroupQO.RemoveValidView.class }) StoreGroupQO storeGroupQO) {
        ServiceResponse<String> result = storeGroupApiService.remove(storeGroupQO.getStoreGroupId());
        return OperationResultUtils.operationResult(
                result, OperationResultUtils.LOG_DELETE+ "门店分组", OperationResultUtils.getOperationObject(result));
    }

    private StoreGroupDTO transferQoToDTO(StoreGroupQO storeGroupQO) {
        StoreGroupDTO storeGroupDTO = EntityUtil.changeEntityNew(StoreGroupDTO.class, storeGroupQO);
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        storeGroupDTO.setEnterpriseId(enterpriseId);
        return storeGroupDTO;
    }
}
